=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2007-03-29 06:55:55 +0000
+++ loggerhead/controllers/atom_ui.py	2007-06-27 17:06:57 +0000
@@ -35,7 +35,7 @@
         h = self._branch.get_history()
         pagesize = int(self._branch.config.get('pagesize', '20'))
 
-        revid_list, start_revid = h.get_file_view(None, None)
+        revid_list = h.get_file_view(h.last_revid, None)
         entries = list(h.get_changes(list(revid_list)[:pagesize]))
 
         vals = {

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2007-09-03 07:31:58 +0000
+++ loggerhead/controllers/changelog_ui.py	2007-09-04 00:34:32 +0000
@@ -87,11 +87,13 @@
         # tell the template to show committer info in the "details block"
         # instead of on each line.
         all_same_author = True
-        author = changes[0].author
-        for e in changes[1:]:
-            if e.author != author:
-                all_same_author = False
-                break
+
+        if changes:
+            author = changes[0].author
+            for e in changes[1:]:
+                if e.author != author:
+                    all_same_author = False
+                    break
 
         vals = {
             'branch': self._branch,

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2007-03-14 06:10:22 +0000
+++ loggerhead/controllers/inventory_ui.py	2007-06-27 16:52:56 +0000
@@ -56,13 +56,19 @@
         if len(args) > 0:
             revid = h.fix_revid(args[0])
         else:
-            revid = None
+            revid = h.last_revid
         
-        file_id = kw.get('file_id', None)
+        try:
+            rev = h.get_revision(revid)
+            inv = h.get_inventory(revid)
+        except:
+            self.log.exception('Exception fetching changes')
+            raise InternalError('Could not fetch changes')
+
+        file_id = kw.get('file_id', inv.root.file_id)
         sort_type = kw.get('sort', None)
 
         try:
-            revid_list, revid = h.get_file_view(revid, file_id)
             rev = h.get_revision(revid)
             inv = h.get_inventory(revid)
         except:
@@ -98,7 +104,7 @@
             'path': path,
             'updir': updir,
             'updir_file_id': updir_file_id,
-            'filelist': h.get_filelist(inv, path, sort_type),
+            'filelist': h.get_filelist(inv, file_id, sort_type),
             'history': h,
             'posixpath': posixpath,
             'navigation': navigation,

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2007-09-03 21:14:32 +0000
+++ loggerhead/history.py	2007-09-04 00:34:32 +0000
@@ -426,24 +426,22 @@
     @with_branch_lock
     def get_file_view(self, revid, file_id):
         """
-        Given an optional revid and optional path, return a (revlist, revid)
-        for navigation through the current scope: from the revid (or the
-        latest revision) back to the original revision.
+        Given a revid and optional path, return a (revlist, revid) for
+        navigation through the current scope: from the revid (or the latest
+        revision) back to the original revision.
         
         If file_id is None, the entire revision history is the list scope.
-        If revid is None, the latest revision is used.
         """
         if revid is None:
             revid = self._last_revid
         if file_id is not None:
-            # since revid is 'start_revid', possibly should start the path tracing from revid... FIXME
+            # since revid is 'start_revid', possibly should start the path
+            # tracing from revid... FIXME
             revlist = list(self.get_short_revision_history_by_fileid(file_id))
             revlist = list(self.get_revids_from(revlist, revid))
         else:
             revlist = list(self.get_revids_from(None, revid))
-        if revid is None:
-            revid = revlist[0]
-        return revlist, revid
+        return revlist
     
     @with_branch_lock
     def get_view(self, revid, start_revid, file_id, query=None):
@@ -470,19 +468,23 @@
         file_id and query are never changed so aren't returned, but they may
         contain vital context for future url navigation.
         """
+        if start_revid is None:
+            start_revid = self._last_revid
+
         if query is None:
-            revid_list, start_revid = self.get_file_view(start_revid, file_id)
+            revid_list = self.get_file_view(start_revid, file_id)
             if revid is None:
                 revid = start_revid
             if revid not in revid_list:
                 # if the given revid is not in the revlist, use a revlist that
                 # starts at the given revid.
-                revid_list, start_revid = self.get_file_view(revid, file_id)
+                revid_list= self.get_file_view(revid, file_id)
+                start_revid = revid
             return revid, start_revid, revid_list
         
         # potentially limit the search
-        if (start_revid is not None) or (file_id is not None):
-            revid_list, start_revid = self.get_file_view(start_revid, file_id)
+        if file_id is not None:
+            revid_list = self.get_file_view(start_revid, file_id)
         else:
             revid_list = None
 
@@ -767,13 +769,16 @@
             old_lines = old_tree.get_file_lines(fid)
             new_lines = new_tree.get_file_lines(fid)
             buffer = StringIO()
-            try:
-                bzrlib.diff.internal_diff(old_path, old_lines,
-                                          new_path, new_lines, buffer)
-            except bzrlib.errors.BinaryFile:
+            if old_lines != new_lines:
+                try:
+                    bzrlib.diff.internal_diff(old_path, old_lines,
+                                              new_path, new_lines, buffer)
+                except bzrlib.errors.BinaryFile:
+                    diff = ''
+                else:
+                    diff = buffer.getvalue()
+            else:
                 diff = ''
-            else:
-                diff = buffer.getvalue()
             out.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid, chunks=self._process_diff(diff)))
         
         return out
@@ -858,23 +863,17 @@
                 m.sbs_chunks = _make_side_by_side(m.chunks)
     
     @with_branch_lock
-    def get_filelist(self, inv, path, sort_type=None):
+    def get_filelist(self, inv, file_id, sort_type=None):
         """
         return the list of all files (and their attributes) within a given
         path subtree.
         """
-        while path.endswith('/'):
-            path = path[:-1]
-        if path.startswith('/'):
-            path = path[1:]
-        
-        entries = inv.entries()
-        
+
+        dir_ie = inv[file_id]
+        path = inv.id2path(file_id)
         file_list = []
-        for filepath, entry in entries:
-            if posixpath.dirname(filepath) != path:
-                continue
-            filename = posixpath.basename(filepath)
+
+        for filename, entry in dir_ie.children.iteritems():
             pathname = filename
             if entry.kind == 'directory':
                 pathname += '/'
@@ -888,18 +887,20 @@
 
             change = util.Container(date=timestamp,
                                     revno=self.get_revno(revid))
-            
-            file = util.Container(filename=filename, executable=entry.executable, kind=entry.kind,
-                                  pathname=pathname, file_id=entry.file_id, size=entry.text_size, revid=revid, change=change)
+
+            file = util.Container(
+                filename=filename, executable=entry.executable, kind=entry.kind,
+                pathname=pathname, file_id=entry.file_id, size=entry.text_size,
+                revid=revid, change=change)
             file_list.append(file)
-        
-        if sort_type == 'filename':
+
+        if sort_type == 'filename' or sort_type is None:
             file_list.sort(key=lambda x: x.filename)
         elif sort_type == 'size':
             file_list.sort(key=lambda x: x.size)
         elif sort_type == 'date':
             file_list.sort(key=lambda x: x.change.date)
-        
+
         parity = 0
         for file in file_list:
             file.parity = parity

=== added file 'loggerhead/history2.py'
--- loggerhead/history2.py	1970-01-01 00:00:00 +0000
+++ loggerhead/history2.py	2007-06-25 10:16:34 +0000
@@ -0,0 +1,194 @@
+"""Extract data suitable for web presentation from Bazaar branches.
+
+The module is the interface between Loggerhead and bzrlib.  Other code in
+Loggerhead should not use bzrlib APIs, and functions and methods in this
+module should not return bzrlib data types.
+
+The core method is `History.getRevisionInfo`.  This method is meant to be
+cheap, so you should not worry about calling it multiple times with the same
+argument or calling it just to retreive a subset of the information it
+returns.
+
+Revisions are described using good old bzrlib revision ids.
+
+Revision ids, file ids, file names and file paths are always utf-8 encoded 8
+bit strings.
+"""
+
+class FileDelta(object):
+    """Information about how an inventory entry changed in a revision.
+
+    All attribute names are fairly self explanatory.  The ``*_parent`` fields
+    are the file id of the parent or ``None`` for the root.
+
+    :ivar file_id:
+    :ivar old_path:
+    :ivar new_path:
+    :ivar old_name:
+    :ivar new_name:
+    :ivar old_parent:
+    :ivar new_parent:
+    :ivar old_kind:
+    :ivar new_kind:
+    :ivar content_change:
+    :type content_change: ``bool``
+    :ivar execute_change: A boolean reflecting the new state if it changed, or
+                          ``None`` if it did not.
+    :type execute_change: ``bool`` or ``None``
+    """
+
+
+class FileChanges(object):
+    """Contains information about which files changed in a revision.
+
+    Note that the 'files changed' information is relative to left-most parent
+    by default, as in 'bzr log -v'.
+
+    :ivar deltas: A list of `FileDelta` objects.
+    :ivar fileset: ``set(d.file_id for d in self.deltas)``, but cached.
+    """
+
+class RevisionInfo(object):
+    """Contains all the immutable information Loggerhead needs about a
+    particular revision.
+
+    :ivar revid: Obvious.
+    :ivar date: The date and time this revision was committed.
+    :type date: ``datetime.datetime``
+    :ivar committer: The committer.
+    :type committer: utf-8 encoded ``str``
+    :ivar revprops: The branch properties.
+    :type revprops: A dictionary mapping ``str``\ s to ``str``\ s (both utf-8
+                    encoded).
+    :ivar message: The commit message of this revision.
+    :type message: utf-8 encoded ``str``
+    :ivar parents: The list of the revids of the revision's parents.
+    """
+
+class BranchRevisionInfo(object):
+    """Contains all the mutable information Loggerhead needs about a
+    particular revision, and points to the immutable data.
+
+    :ivar revno: The dotted revno of this revision in this branch.
+    :type revno: ``str``
+    :ivar where_merged: The list of revids that have this revision as a
+                        parent.
+    :ivar info: The corresponding `RevisionInfo`.
+    """
+
+class FileEntry(object):
+    """An entry in the list returned by `getFileList`.
+
+    :ivar path: The path of the object in the revision passed to
+                `getFileList`.
+    :ivar fileid: Obvious.
+    :ivar kind: one of ``'file'``, ``'executable'``, ``'link'`` or
+                ``'directory'``.
+    :type kind: ``str``
+    :ivar last_changed_revid: The revid of the revision in which this path
+                              last changed.  Not that this is recursive,
+                              i.e. the last_changed_revid for a directory is
+                              the revid in which the directory or anything
+                              contained in in changed.x
+    :ivar size: The size of this object in this revision in bytes.
+    :type size: ``int``
+    """
+
+class History(object):
+    """Provide the information loggerhead needs about a bzrlib Branch.
+
+    An instance of this class is effectively a wrapper around a
+    `bzrlib.branch.Branch` that translates the information provided by bzrlib
+    into a form convenient for use in loggerhead.
+
+    :ivar last_revision: The revid of the tip of the branch.
+    """
+
+    @classmethod
+    def fromBranch(cls, branch):
+        """Create and initialize a `History` object from a
+        `bzrlib.branch.Branch`.
+        """
+
+    def outOfDate(self):
+        """Decide whether this History object is still current.
+
+        At least to start with, an out of date History object should be thrown
+        away and a fresh one created.
+        """
+
+    def getRevisionInfo(self, revid):
+        """Find the `BranchRevisionInfo` object for a given revision id.
+
+        This operation is to be thought of as 'cheap', which is to say that
+        you should not worry about calling it repeatedly with the same
+        argument or about calling it just to retrieve a subset of the
+        information it includes.
+
+        :param revid: a revision id.
+        :returns: a `BranchRevisionInfo` object or ``None`` if the given
+                  revision id does not exist in the branch.
+        """
+
+    def getChanges(self, revid, other_revid=None):
+        """Find the `FileChanges` object between two revision.
+
+        :param revid: A revision id.
+        :param other_revid: The revision id to compare against.  If ``None``
+                            or not given, compare against the leftmost parent
+                            of ``revid``.
+        :returns: The corresponding `FileChanges` object.
+        """
+
+    # There will need to be some more methods here to allow searching for
+    # revisions by date, committer, commit message, etc...
+
+    def getDiff(self, revidA, revidB):
+        """Compute the diff between two revisions.
+
+        Not sure what this will return yet, probably something similar to but
+        more structured than the 'chunks' loggerhead already works with.
+        """
+
+    def getDirList(self, fileid, revid):
+        """List the files in the given directory as of the given revision.
+
+        :return: a list of `FileEntry` objects.
+        """
+
+    def getFileAnnotation(self, fileid, revid):
+        """Annotate the given file.
+
+        Not sure what this should return, mostly likely a list of
+        (revid-or-None, line contents) (and ``None`` for a binary file).
+        """
+
+    def getFileText(self, fileid, revid):
+        """Fetch the contents of a file at a particular revision.
+
+        :returns: (filename, data)
+        :rtype: ``(str, str)``
+        """
+
+    def getBundle(self, revid, compare_revid=None):
+        """Return a bundle for this revision.
+
+        Relative to compare_revid, or left-most parent if this is not supplied
+        or None.
+
+        :return: A bundle (in a string).
+        """
+
+    def normalizeRevivionSpec(self, revspec):
+        """Convert a revid or revno or 'head' to a revid.
+
+        :returns: a revision id.
+        """
+
+    def normalizeFileArguments(self, revid, fileid, filepath):
+        """Handle various ways of specifying a file in the branch.
+
+        One of ``fileid`` and ``filepath`` should be None.
+
+        :returns: (fileid, filepath)
+        """

=== modified file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2007-09-03 07:49:54 +0000
+++ loggerhead/static/css/style.css	2007-09-04 00:34:32 +0000
@@ -400,7 +400,7 @@
 .diff-option-buttons .collapse-file-showall { display: none; }
 
 .diff-block .show-button { display: none; }
-.diff .collapse-style-unified-content { display: none; }
+.collapse-style-unified-content { display: none; }
 .collapse-style-sbs-show { display: none; }
 
 .diff-insert {

=== renamed file 'loggerhead/tests/test_upgrade.py' => 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_upgrade.py	2007-06-19 13:52:22 +0000
+++ loggerhead/tests/test_corners.py	2007-06-27 15:38:47 +0000
@@ -4,9 +4,9 @@
 from turbogears import testutil
 import cherrypy
 
-class TestSurviveOverUpgrade(BasicTests):
+class TestCornerCases(BasicTests):
 
-    def test_upgrade(self):
+    def test_survive_over_upgrade(self):
         self.createBranch()
 
         f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
@@ -29,3 +29,25 @@
 
         testutil.create_request('/project/branch/changes')
         assert msg in cherrypy.response.body[0]
+
+    def test_revision_only_changing_execute_bit(self):
+        self.createBranch()
+
+        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
+        try:
+            f.write("foo")
+        finally:
+            f.close()
+        self.tree.add('myfilename')
+        msg = 'a very exciting commit message'
+        self.tree.commit(message=msg)
+
+        os.chmod(os.path.join(self.bzrbranch, 'myfilename'), 0755)
+
+        newrevid = self.tree.commit(message='make something executable')
+
+        self.setUpLoggerhead()
+
+
+        testutil.create_request('/project/branch/revision/'+newrevid)
+        assert 'executable' in cherrypy.response.body[0]

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2007-06-22 08:04:22 +0000
+++ loggerhead/tests/test_simple.py	2007-06-26 19:23:08 +0000
@@ -89,7 +89,7 @@
 
     def test_changes(self):
         testutil.create_request('/project/branch/changes')
-        assert self.msg in cherrypy.response.body[0]
+        assert cgi.escape(self.msg) in cherrypy.response.body[0]
 
     def test_changes_search(self):
         testutil.create_request('/project/branch/changes?q=foo')
@@ -105,6 +105,10 @@
         testutil.create_request('/project/branch/files')
         assert 'myfilename' in cherrypy.response.body[0]
 
+    def test_revision(self):
+        testutil.create_request('/project/branch/revision/' + self.revid)
+        assert 'myfilename' in cherrypy.response.body[0]
+
 class TestWithSimpleTreeAndCache(TestWithSimpleTree):
     config_template = """
     testing = True

